/************************************************/
/*						*/
/*      Copyright:				*/
/*		 Jean-Marc Pigeon <jmp@safe.ca>	*/
/*	Distributed under the Gnu Public	*/
/*	License, see the License file in this	*/
/*	package.				*/
/*						*/
/*	GESPBS:					*/
/*	Take care off Pop_Before_Smtp function.	*/
/*						*/
/************************************************/
#include	<errno.h>
#include	<unistd.h>
#include	<string.h>
#include	"subrou.h"
#include	"unilck.h"
#include	"unieml.h"
#include	"gespbs.h"

/*file to record POP/IMAP before SMTP request	*/
#define	PIBSMTP	"pibsmtp"
#define	POPRELY	"poprelay"

#define	PIBFRM	"%ld %s %s\n"

#define	TVAL	120	/*Entry validity time	*/
/*
^L
*/
/************************************************/
/*						*/
/*	Procedure to transfert recent POP/IMAP	*/
/*	access information to poprelay file	*/
/*						*/
/************************************************/
static void dropclean(time_t curtime)

{
#define	ODDP	"gespbs.c:dropclean,"

#define	BGLINE	500

FILE *fpibnew;
FILE *fpiblst;
FILE *fpopnew;
int phase;
int proceed;
char pibnew[300];
char piblst[300];
char popnew[300];
char poprly[300];
char line[BGLINE];

fpibnew=(FILE *)0;
fpiblst=(FILE *)0;
fpopnew=(FILE *)0;
phase=0;
proceed=true;
while (proceed==true) {
  switch (phase) {
    case   0	:	/*getting filename	*/
      (void) rou_mkname(pibnew,sizeof(pibnew),d_etc,"",PIBSMTP,".new");
      (void) rou_mkname(piblst,sizeof(piblst),d_etc,"",PIBSMTP,".lst");
      (void) rou_mkname(popnew,sizeof(popnew),d_etc,"",POPRELY,".new");
      (void) rou_mkname(poprly,sizeof(poprly),d_etc,"",POPRELY,"");
      break;
    case   1	:	/*opening file		*/
      if ((fpiblst=fopen(piblst,"r"))==(FILE *)0)
	phase=9999;	/*file missing, get out	*/
      else {
	if ((fpibnew=fopen(pibnew,"w"))==(FILE *)0) {
	  (void) fclose(fpiblst);
	  phase=9999;
	  (void) alert(0,"%s Unable to open <%s> (error=<%s>)",
			  ODDP,pibnew,strerror(errno));
          }
	else {
	  if ((fpopnew=fopen(popnew,"w"))==(FILE *)0) {
	    (void) fclose(fpiblst);
	    (void) fclose(fpibnew);
	    phase=9999;
	    (void) alert(0,"%s Unable to open <%s> (error=<%s>)",
			    ODDP,popnew,strerror(errno));
	    }
	  }
	}
      break;
    case   2	:	/*transferring file	*/
      while (fgets(line,sizeof(line)-1,fpiblst)!=(char *)0) {
	time_t stamp;
	char ip[BGLINE];
	char username[BGLINE];

        if (sscanf(line,PIBFRM,&stamp,ip,username)==3) {
	  if ((stamp+TVAL)>curtime) {
	    (void) fprintf(fpibnew,"%s",line);
	    (void) fprintf(fpopnew,"%s/32\n",ip);
	    }
          }
	}
      break;
    case   3	:	/*closing files		*/
      (void) fclose(fpopnew);
      (void) fclose(fpibnew);
      (void) fclose(fpiblst);
      break;
    case   4	:	/*moving PIBSMTP file	*/
      (void) unlink(piblst);
      (void) link(pibnew,piblst);
      (void) unlink(pibnew);
      break;
    case   5	:	/*moving POPRELY file	*/
      (void) unlink(poprly);
      (void) link(popnew,poprly);
      (void) unlink(popnew);
      break;
    default	:	/*SAFE guard		*/
      proceed=false;
      break;
    }
  phase++;
  }
}
/*
^L
*/
/************************************************/
/*						*/
/*	Procedure to add an entry to the pibsmt	*/
/*	file.					*/
/*						*/
/************************************************/
void pbs_addentry(char *username,char *ip)

{
if (lck_locking(PIBSMTP,LCK_LOCK,5)==true) {
  FILE *fichier;
  char filename[300];

  (void) rou_mkname(filename,sizeof(filename),d_etc,"",PIBSMTP,".lst");
  if ((fichier=fopen(filename,"a"))!=(FILE *)0) {
  /* "USER" and "IP" are 2 env values provided by 'dovecot' calling   */
    if ((username!=(char *)0)&&(ip!=(char *)0)) {
      (void) fprintf(fichier,PIBFRM,rou_systime(),ip,username);
      }
    (void) fclose(fichier);
    }
  else
    (void) tracing(TRALOG,0,"%s Unable to open file <%s> (status=<%s>)",
			     appname,filename,strerror(errno));
  (void) lck_locking(PIBSMTP,LCK_UNLOCK,1);
  }
}
/*
^L
*/
/************************************************/
/*						*/
/*	Procedure to check if file PIBSMTP was	*/
/*	change or due for a little old pop	*/
/*	access list cleaning.			*/
/*						*/
/************************************************/
void pbs_cleanentry()

{
static time_t lastclean=(time_t)0;

int letsdoit;
time_t curtime;
char filename[300];

letsdoit=false;
curtime=rou_systime();
(void) rou_mkname(filename,sizeof(filename),d_etc,"",PIBSMTP,".lst");
switch (rou_isnewfile(filename,lastclean)) {
  case  -1	:	/*input file missing	*/
    break;		
  case   0	:	/*file was not updated	*/
    if ((lastclean+180)<curtime)
      letsdoit=true;
    break;		
  case	1	:	/*file has been updated	*/
    letsdoit=true;
    break;
  }
if (letsdoit==true) {
  if (lck_locking(PIBSMTP,LCK_LOCK,2)==true) {
    (void) dropclean(curtime);
    lastclean=curtime;
    (void) lck_locking(PIBSMTP,LCK_UNLOCK,1);
    }
  }
}
