/************************************************/
/*						*/
/*      Copyright:				*/
/*		 Jean-Marc Pigeon <jmp@safe.ca>	*/
/*	Distributed under the Gnu Public	*/
/*	License, see the License file in this	*/
/*	package.				*/
/*						*/
/*	MODCHK:					*/
/*	Module level routines to handle		*/
/*	E-mail RCPT request by checking	with	*/
/*	local sender and remote Mentor.		*/
/*						*/
/************************************************/
#include	<stdlib.h>
#include	<unistd.h>
#include	<malloc.h>
#include	<syslog.h>
#include	<string.h>
#include	<errno.h>
#include	<string.h>
#include	<signal.h>
#include	"subrou.h"
#include	"unieml.h"
#include	"unidns.h"
#include	"unicac.h"
#include	"gesque.h"
#include	"mngusr.h"
#include	"modchk.h"
typedef	enum	{	/*message type		*/
	msg_ckrcpt,	/*check RCPT info	*/
	msg_ckrsnd,	/*RCPT info (via sender)*/
	msg_ckrrly,	/*RCPT info (via relay)	*/
	msg_ckrmnt,	/*RCPT info (via mentor)*/
	msg_advice,	/*Requesting Mentor adv	*/
	msg_uniqid,	/*sending Uniqid	*/
	msg_refresh,	/*send refresh status	*/
	msg_sent,	/*sent acknowledge	*/
	msg_unknown	/*unknown message	*/
	}MENUM;

typedef struct {	/*adv dispatcher struct	*/
	pid_t chkpid;	/*sending channel pid	*/
	char *msg;	/*message to send	*/
	}DISTYP;

typedef struct	{	/*checker list		*/
	char *chkname;
	char *lst;
	}SNDTYP;
/*

*/
/************************************************/
/*						*/
/*	routine to locate one SNDTYP among an	*/
/*	SNDTYP list.				*/
/*						*/
/************************************************/
SNDTYP *findsentlst(SNDTYP **lst,char *chkname)

{
SNDTYP *found;

found=(SNDTYP *)0;
if (lst!=(SNDTYP **)0) {
  int i;

  for (i=0;lst[i]!=(SNDTYP *)0;i++) {
    if (strcmp(lst[i]->chkname,chkname)==0) {
      found=lst[i];
      break;
      }
    }
  }
return found;
}
/*

*/
/************************************************/
/*						*/
/*	routine to extract the message code	*/
/*						*/
/************************************************/
MENUM getmsgcode(char *message)

{
static	struct	{
	MENUM code;	/*message CODE		*/
	char *key;
	}msgcode[]={
		{msg_ckrcpt,CKRCPT},
		{msg_ckrsnd,CKRSND},
		{msg_ckrrly,CKRRLY},
		{msg_ckrmnt,CKRMNT},
		{msg_advice,ADVICE},
		{msg_uniqid,UNIQID},
		{msg_refresh,REFRESH},
		{msg_sent,SENT},
		{msg_unknown,(char *)0}
		};

MENUM code;
int i;

code=msg_unknown;
for (i=0;msgcode[i].key!=(char *)0;i++) {
  if (strstr(message,msgcode[i].key)==message) {
    char *ptr;

    if ((ptr=strchr(message,' '))!=(char *)0) {
      ptr++;
 
      (void) strcpy(message,ptr);
      code=msgcode[i].code;
      break;
      }
    }
  }
return code;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to locate a dispatch channel	*/
/*	previously created to request advice.	*/
/*						*/
/************************************************/
static DISTYP *getdispatch(DISTYP **dlst,pid_t chkpid)

{
DISTYP *found;

found=(DISTYP *)0;
if (dlst!=(DISTYP **)0) {
  int i;

  for (i=0;dlst[i]!=(DISTYP *)0;i++) {
    if (dlst[i]->chkpid==chkpid) {
      found=dlst[i];
      break;
      }
    }
  }
return found;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to wait response from remote	*/
/*	checker.				*/
/*						*/
/************************************************/
static char *gettlsline(char *line,int taille,CTTYP *contact)

{
char *ptr;

line[0]='\000';
if ((ptr=tls_fgets(line,taille,contact->tls))==(char *)0) {
  switch(tls_waitforchar(contact->tls,1000)) {
    case -1		:
      switch(errno) {
	case EINTR	:	/*received a signal, lets see...*/
	  break;		/*must be an IPC messages	*/
	default		:	/*hummm real code fault report	*/
	  (void) code_fault("%s select error '%s'",
			    "modchk.c:gettlsline",strerror(errno));
	  break;
	}
      break;
    case  0		:
     				/*standard time out		*/
      break;
    default		:
      if (tls_getfeed(contact->tls)==true)
        ptr=tls_fgets(line,taille,contact->tls);
      break;
    }
  }
if (ptr!=(char *)0) {
  (void) que_cleanstring(line);
  }
return ptr;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to relay an TCP message to	*/
/*	to the IPC channel.			*/
/*						*/
/************************************************/
static void relaytoipc(BOXTYP *boxe,char **msg)

{
#define	OPRL	"modchk.c:relaytoipc,"

char *msgref;
int phase;
int proceed;

msgref=0;
phase=0;
proceed=true;
while (proceed==true) {
  switch(phase) {
    case 0	:
      if (rou_vecsize((void **)msg)<3) {
	int num;
	int i;

	(void) alert(0,"%s message too small",OPRL);
	num=rou_vecsize((void **)msg);
	for (i=0;i<num;i++)
	  (void) alert(0,"%s message[%d]=<%s>",OPRL,i,msg[i]);
	phase=999;
	}
      break;
    case 1	: 
      if (strstr(msg[0],ANSWID)!=msg[0]) {
	(void) alert(0,"%s Wrong message ID <%s>",msg[0]);
	phase=999;
	}
      break;
    case 2	: 
      if (msg!=(char **)0) {
        int i;	
        char *ipcmsg;

	ipcmsg=(char *)0;
	(void) strcpy(msg[0],msg[0]+strlen(ANSWID));
        for (i=1;msg[i]!=(char *)0;i++) {
          if ((msg[i][0]=='.')&&(msg[i][1]=='\000'))
	    break;
	  ipcmsg=rou_memstrcat(ipcmsg,"%s\n",msg[i]);
          }
        if (ipc_postreply(boxe,msg[0],ipcmsg)==0) {
	  (void) alert(0,"%s Unable to post, msgref=<%s> msg=<%s> (error=%s)",
			 OPRL,msg[0],ipcmsg,strerror(errno));
          }
        }
      break;
    default	:	/*SAFE guard		*/
      proceed=false;
      break;
    }
  phase++;
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to send an CKRCPT request to	*/
/*	mentor type server.			*/
/*						*/
/************************************************/
static void sendreqst(CTTYP *cta,char *msgref,char *command,char *rqst)

{
char *info;

info=rou_memstrcat((char *)0,"%s %s\n%s %s\n.",RQSTID,msgref,command,rqst);
(void) eml_printfcrlf(cta->tls,info);
info=rou_freestr(info);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to open a connexion to a	*/
/*	sender, to use to check if user is	*/
/*	existing or not				*/
/*						*/
/************************************************/
static CHKTYP *chkopenauxi(SKRTYP *skr,char *uniqid,CHKTYP *checker)

{
/*time to retry to connect to auxiliary		*/
#define	MXRTRY	300	

if (checker==(CHKTYP *)0) {
  static time_t lasttry=(time_t)0;
   
  time_t curtime;
  curtime=rou_systime();
  if ((lasttry+MXRTRY)<curtime) {
    checker=eml_openauxiliary(skr,uniqid);
    lasttry=curtime;
    }
  }
return checker;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to update a remote mentor	*/
/*	with a user status			*/
/*						*/
/************************************************/
static void refreshmentor(BOXTYP *boxe,char *caller,pid_t mnt,
			  RCPTMSG *rm,int chkstatus)

{
char *locmsg;

locmsg=rou_memstrcat((char *)0,"%s %s %s %s %s,%d %d %d",REFRESH,
				rm->from,rm->iprem,
				rm->rcpt,rm->listenip,rm->listenport,
				chkstatus,rm->relais);
(void) tracing(TRALOG,0,"JMPDBG refreshmentor=<%s>",locmsg);
if (ipc_relaymsg(boxe,caller,mnt,locmsg)==0) 
  (void) alert(0,"%s Unable to relaymsg <%s> to mentor",
		 "modchk.c:refreshmentor",locmsg);
locmsg=rou_freestr(locmsg);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to send default value to	*/
/*	advice request while not handle by a	*/
/*	Mentor.					*/
/*						*/
/************************************************/
static int defltadv(BOXTYP *boxe,char *caller,ADVIMSG *advimsg,int slice)

{
if (advimsg!=(ADVIMSG *)0) {
  if (advimsg->doms!=(DOMCHK **)0) {
    char *advrly;
    int i;
    int itspam;

    advrly=(char *)0;
    itspam=false;
    if (advimsg->relayable==false)
      if (advimsg->spmlvl>=advimsg->spmlim)
        itspam=true;
    for (i=0;advimsg->doms[i]!=(DOMCHK *)0;i++) {
      if (advimsg->doms[i]->forwarded==false) {
	char *dmsg;

	if ((dmsg=eml_domtoadv(advimsg->doms[i],itspam))!=(char *)0) {
	  advrly=rou_memstrcat(advrly,"%s",dmsg);
	  dmsg=rou_freestr(dmsg);
	  }
	}
      }
    if (advrly!=(char *)0) {
      slice++;
      advrly=rou_memstrcat(advrly,"%d %d/%d This was my advice",
				  CMDOK, slice,slice);
      (void) ipc_postreply(boxe,caller,advrly);
      advrly=rou_freestr(advrly);
      }
    }
  }
return slice;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to handle a check by relay	*/
/*	request, purpose is to check if the 	*/
/*	orignator is an acceptable recipient	*/
/*	from the relay SMTP daemon as the domain*/
/*	is unknown from clement while the flag	*/
/*	Relay is set.				*/
/*						*/
/************************************************/
static char *checkbyrelay(BOXTYP *boxe,CHKTYP *rely,char *caller,RCPTMSG *rm)

{
char *answ;
int chkstatus;
int phase;
int proceed;

answ=(char *)0;
chkstatus=ERRPROC;
phase=0;
proceed=true;
while (proceed==true) {
  switch (phase) {
    case 0	:	/*does relay accept?	*/
      if (rely!=(CHKTYP *)0)
        chkstatus=usr_isexisting(rely,caller,rm);
      break;
    default	:	/*SAFE guard		*/
      if (answ==(char *)0) {
	PLDTYP *pld;

        if ((pld=cac_getentry(rm->rcpt,pldsmstat))!=(PLDTYP *)0) {
	  answ=rou_memstrcat((char *)0,"%d %s",pld->info.smstat.status,
					       pld->info.smstat.str);
	  }
        else
          answ=strdup(eml_ascstat(chkstatus,rm->rqst));
	}
      proceed=false;
      break;
    }
  phase++;
  }
return answ;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to handle a check by sender	*/
/*	request, purpose is to check if the 	*/
/*	orignator is an acceptable recipient	*/
/*	from the sender stand point.		*/
/*						*/
/************************************************/
static char *checkbysndr(BOXTYP *boxe,CHKTYP *sndr,char *caller,RCPTMSG *rm)

{
#define	OPKE	"mkdochk.c:checkbysndr,"

char *answ;
pid_t mnt;
pid_t relaysndr;
char *pids;
int chkstatus;
int phase;
int proceed;

answ=(char *)0;
mnt=(pid_t)0;
relaysndr=(pid_t)0;
pids=(char *)0;
chkstatus=ERRPROC;
phase=0;
proceed=true;
while (proceed==true) {
  switch (phase) {
    case  0 	:	/*checkin sender available		*/
      if (sndr==(CHKTYP *)0) {
	answ=strdup(eml_ascstat(NOSNDR));
	proceed=false;
        }
      break;
    case  1 	:	/*extracting checker and relaysender	*/
      if ((pids=strchr(rm->rqst,'|'))!=(char *)0) {
	*pids='\000';
	pids++;
	(void) sscanf(pids,"%d %d",&mnt,&relaysndr);
        }
      break;

    case  2 	:	/*relay requested?			*/
      if (rm->relais==true)
	phase++;
      break;

    case  3 	:	/*no relay, lest check MX		*/
      if (eml_locatedomain(sndr->doms,eml_strdomain(rm->rcpt))==false) {
        if (dns_isgoodmx(eml_strdomain(rm->rcpt),rm->listenip)==false) {
	  answ=strdup(eml_ascstat(NORELAY));
	  proceed=false;
	  }
	}
      break;

    case  4 	:	/*checking by sender			*/
      if (sndr!=(CHKTYP *)0)
        chkstatus=usr_isexisting(sndr,caller,rm);
      break;

    case  5	:	/*user is not good one, is it relayable?*/
      if (chkstatus!=CMDOK) {
	if ((rm->relais==true)&&(relaysndr!=(pid_t)0)) {
	  char *locmsg;
  
          locmsg=rou_memstrcat((char *)0,"%s %s",CKRRLY,rm->rqst);
          if (ipc_relaymsg(boxe,caller,relaysndr,locmsg)==0) 
            answ=strdup(eml_ascstat(ERRPROC,locmsg));
	  locmsg=rou_freestr(locmsg);
          proceed=false;/*lets relay process to take care	*/
	  }
	}
      break;

    case  6	:	/*do we have a sender ?			*/
      if (mnt==(pid_t)0)/*No					*/
	phase=999;	/*lets get out fast			*/
      break;
	
    case  7	:	/*We have a sender defined		*/
      switch (chkstatus) {
	case CMDOK  :	/*lets update the mentor user status	*/
	case BADPAR :
	case REJECT :
          (void) refreshmentor(boxe,caller,mnt,rm,chkstatus);
	  break;
	default	    :
	  break;
	}
      break;

    case  8	:	/*lets get advice from mentor		*/
      if (chkstatus==CMDOK) {
	char *locmsg;

        locmsg=rou_memstrcat((char *)0,"%s %s",CKRMNT,rm->rqst);
        if (ipc_relaymsg(boxe,caller,mnt,locmsg)==0) 
	  (void) alert(0,"%s Unable to relaymsg <%s> to mentor",OPKE,locmsg);
	else
	  proceed=false;/*Mentor will answer			*/
	locmsg=rou_freestr(locmsg);
	}
      break;

    default	:	/*SAFE guard				*/
      if (answ==(char *)0) {
	PLDTYP *pld;

        if ((pld=cac_getentry(rm->rcpt,pldsmstat))!=(PLDTYP *)0) {
	  answ=rou_memstrcat((char *)0,"%d %s",pld->info.smstat.status,
					       pld->info.smstat.str);
	  }
        else
          answ=strdup(eml_ascstat(chkstatus,rm->rqst));
	}
      proceed=false;
      break;
    }
  phase++;
  }
return answ;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to handle a check by Mentor	*/
/*	request, purpose is to check if the 	*/
/*	orignator is an acceptable recipient	*/
/*	from the Mentor stand point.		*/
/*						*/
/************************************************/
static char *checkbymntr(BOXTYP *boxe,CHKTYP *mntr,char *caller,char *rqst)

{
char *answ;
int phase;
int proceed;

answ=(char *)0;
phase=0;
proceed=true;
while (proceed==true) {
  switch (phase) {
    case  0	:	/*do we have a mentor?	*/
      if (mntr==(CHKTYP *)0) {	/*NO!		*/
	phase=999;	/*get out fast		*/
	}
      break;
    case  1	:	/*Yes we have a Mentor	*/
      if ((mntr->contact!=(CTTYP *)0)&&(mntr->contact->tls!=(TLSFILE *)0)) {
        (void) sendreqst(mntr->contact,caller,CKRCPT,rqst);
	proceed=false;
	}
      break;
    default	:	/*SAFE guard		*/
      answ=rou_memstrcat(answ,"%d",CMDOK);
      proceed=false;
      break;
    }
  phase++;
  }
return answ;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to handle recipient checking	*/
/*	process					*/
/*						*/
/************************************************/
static char *checkingby(BOXTYP *boxe,CHKTYP *chkr,char *caller,
		        char *rqst,MENUM code)

{
char *answ;
RCPTMSG *rm;
int phase;
int proceed;

answ=(char *)0;
rm=(RCPTMSG *)0;
phase=0;
proceed=true;
while (proceed==true) {
  switch (phase) {
    case 0	:	/*scanning message	*/
      if ((rm=eml_getrcptmsg(rqst))==(RCPTMSG *)0) {
	answ=strdup(eml_ascstat(BADPAR,rqst));
	proceed=false;
	}
      break;
    case 1	:	/*dispatching message	*/
      switch (code) {
        case msg_ckrsnd		:
          answ=checkbysndr(boxe,chkr,caller,rm);
          break;
        case msg_ckrmnt		:
          answ=checkbymntr(boxe,chkr,caller,rqst);
          break;
        case msg_ckrrly		:
          answ=checkbyrelay(boxe,chkr,caller,rm);
          break;
	default	:
	  (void) alert(0,"modechk.c:checkingby, unexpected code (bug?)");
	  answ=strdup(eml_ascstat(ERRPROC,rqst));
	  break;
        }
      break;
    default	:	/*SAFE guard		*/
      rm=eml_freercptmsg(rm);
      proceed=false;
      break;
    }
  phase++;
  }
return answ;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to rebounce MSMTP request	*/
/*	(CHKRCPT and ADVICE) to the remote	*/
/*	Mentor or if no such capability to a	*/
/*	local checker.				*/
/*						*/
/************************************************/
static int chkrebounce(BOXTYP *boxe,CHKTYP *checker)

{
#define	OPCR	"modchk.c:chkrebounce,"

int gotone;
char *caller;
char *rqst;

gotone=false;
caller=(char *)0;
if ((rqst=ipc_getpost(boxe,&caller))!=(char *)0) {
  char *answ;
  MENUM code;

  answ=(char *)0;
  gotone=true;
  switch (code=getmsgcode(rqst)) {
    case msg_ckrsnd		:
    case msg_ckrmnt		:
    case msg_ckrrly		:
      answ=checkingby(boxe,checker,caller,rqst,code);
      break;
    case msg_ckrcpt		:
      (void) alert(0,"%s unexpected request <%s>",OPCR,rqst);
      answ=strdup(eml_ascstat(ERRPROC));
      break;
    case msg_refresh		:
      if ((checker!=(CHKTYP *)0)&&(checker->control==c_mentor)) {
        (void) sendreqst(checker->contact,caller,REFRESH,rqst);
	}
      break;
    case msg_uniqid		:
      if ((checker!=(CHKTYP *)0)&&(checker->control==c_mentor)) {
	(void) sendreqst(checker->contact,caller,UNIQID,rqst);
	}
      else {
	/*Facking a Mentor Acknowledge answer	*/
        answ=rou_memstrcat(answ,"%d MENTOR PROXY",CMDOK);
	}
      break;
    case msg_advice		:
      if ((checker!=(CHKTYP *)0)&&(checker->control==c_mentor)) {
 	char *advice;
  
        advice=rou_memstrcat((char *)0,"%s %s",ADVICE,rqst);
        (void) eml_sendstream(checker->contact,RQSTID,caller,advice);
        advice=rou_freestr(advice);
	}
      else {
	ADVIMSG *advimsg;

	if ((advimsg=eml_getadvimsg(rqst))!=(ADVIMSG *)0) {
          (void) defltadv(boxe,caller,advimsg,0);
  	  advimsg=eml_freeadvimsg(advimsg);
	  }
	}
      break;
    case msg_sent		:
      if ((checker!=(CHKTYP *)0)&&(checker->control==c_mentor))
	(void) sendreqst(checker->contact,caller,SENT,rqst);
      else
        (void) alert(0,"%s Unexpected sent request <%s>",OPCR,rqst);
      break;
    case msg_unknown		:
    default			:
      (void) alert(0,"%s request <%s> unknown",OPCR,rqst);
      answ=strdup(eml_ascstat(ERRPROC));
      break;
    }
  if (answ!=(char *)0) {
    (void) ipc_postreply(boxe,caller,answ);
    answ=rou_freestr(answ);
    }
  caller=rou_freestr(caller);
  rqst=rou_freestr(rqst);
  }
return gotone;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to handle checking request	*/
/*	submitted according domain list		*/
/*						*/
/************************************************/
static void dochecking(BOXTYP *boxe,SKRTYP *skr)

{
#define OPBI    "modchk.c:dochecking,"
#define KEEPACE	60	/*max idle time		*/

char *uniqid;

if ((uniqid=ipc_getonemsgref(boxe))!=(char *)0) {
  CHKTYP *checker;
  char *strpref;
  char **msg;
  int phase;

  checker=(CHKTYP *)0;
  strpref="Checking";
  if (skr->pref==p_checker) {
    strpref="Mentoring";
    }
  msg=(char **)0;
  phase=0;
  while (quickout==false) {
    char line[800];

    switch (phase) {
      case  0	:	/*starting connexion		*/
	checker=chkopenauxi(skr,uniqid,checker);
	break;
      case 1	:	/*checking if parent still up	*/
	if (rou_checkprocess(getppid())==false) {
          checker=eml_closeauxiliary(checker);
	  quickout=true;
	  }
	break;
      case 2	:	/*getting and forwarding request*/
	if (quickout==false) {
	  TIMESPEC timeout;

	  timeout.tv_sec=0;
	  timeout.tv_nsec=500000;
          if (ipc_waitpost(boxe,&timeout)==true) {
	    while (chkrebounce(boxe,checker)==true);
	    }
	  }
	break;
      case 3	:	/*updating user cache		*/
        (void) cac_expireentries();
	break;
      case 4	:	/*checker not up lets cycle	*/
	if (checker==(CHKTYP *)0) {
          (void) settitle("%s, Waiting for %s from %s",
			   appname,strpref,skr->hostname);
          phase=-1;
	  (void) sleep(1);
	  }
	break;
      case 5	:	/*collecting answer from Mentor*/
        (void) settitle("%s, %s done by %s",appname,strpref,skr->hostname);
	switch (checker->control) {
	  case c_local		:
	    break;
	  case c_standard	: 
	    /*standard SMTP remote, we do not care about input	*/
	    /*here, lets forget about it			*/
	    while (gettlsline(line,sizeof(line),checker->contact)!=(char *)0);
	    break;
	  case c_mentor		:
	    while (gettlsline(line,sizeof(line),checker->contact)!=(char *)0) {
              if ((msg==(char **)0)&&(strstr(line,ANSWID)!=line)) {
                break; /*got a disconnect from mentor */
		}
              msg=(char **)rou_addveclst((void **)msg,(void *)strdup(line));
              if ((strlen(line)==1)&&(line[0]=='.')) {
                (void) relaytoipc(boxe,msg);
                msg=rou_freestrlst(msg);
		break;	/*got all messages		*/
	        }
	      }
	    break;
	  }
	break;
      case 6	:	/*checking idle situation	*/
	phase=0;
	switch (checker->control) {
	  case c_local		:
	    break;
	  case c_standard	:
	  case c_mentor		:
	    if (eml_smtpidle(checker,KEEPACE)==true) {
              if (eml_sendcmd(checker->contact,CMDOK,RSET)!=CMDOK) {
	        phase=999;	/*trouble		*/
		break;
	        }
	      }
            if (tls_iseof(checker->contact->tls)==true)  {
	      phase=999;	/*trouble		*/
	      }
	    break;
	  }
	break;
      default	:	/*lets try again		*/
        checker=eml_closeauxiliary(checker);
        phase=-1;
	break;
      }
    phase++;
    }
  if (checker!=(CHKTYP *)0)
    checker=eml_closeauxiliary(checker);
  }
else {
  (void) alert(0,"%s Unable to get uniqid");
  }
(void) cac_cleanentries();
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to initiate checker/sender	*/
/*	process.				*/
/*						*/
/************************************************/
static pid_t startskr(BOXTYP *boxe,SKRTYP *skr)

{
#define	OPST	"modchk.c:startskr,"
pid_t child;

switch((child=rou_fork(false))) { 
  case  -1	:
    (void) alert(0,"%s, Unable to fork process (error=<%s>)",
		   OPST,strerror(errno));
    break;
  case   0	:	
    (void) dochecking(boxe,skr);
    (void) exit(0);
    break;
  default	:
    break;
  }
return child;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to start or restart a 	*/
/*	checker/sender process.			*/
/*						*/
/************************************************/
static void checkskr(BOXTYP *boxe,SKRTYP **skrlst)

{
if (skrlst!=(SKRTYP **)0) {
  time_t curtime;
  int i;

  curtime=rou_systime();
  for (i=0;skrlst[i]!=(SKRTYP *)0;i++) {
    SKRTYP *sk;

    sk=skrlst[i];
    switch (sk->stat) {
      case s_new	:
      case s_down	:
	if ((sk->lastact+60)<curtime) {
	  sk->lastact=curtime;
	  sk->stat=s_active;
	  sk->skrpid=startskr(boxe,sk);
	  }
	break;
      case s_active	:
	if (kill(sk->skrpid,SIGCHECK)<0) {
	  sk->stat=s_down; 
	  }
	break;
      default		:
        (void) alert(0,"%s Unexpected skr=<%s> stat='%d'",
		        "modchk.c:checkskr,",sk->hostname,sk->stat);
	break;
      }
    }
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to stop all checker/sender	*/
/*	process.				*/
/*						*/
/************************************************/
static void dropskr(SKRTYP **skrlst)

{
int nbr;

if ((nbr=rou_vecsize((void **)skrlst))>0) {
  int i;
  int n;
  pid_t *pidlst;

  pidlst=(pid_t *)calloc(nbr,sizeof(pid_t));
  for (i=0,n=0;skrlst[i]!=(SKRTYP *)0;i++) {
    SKRTYP *sk;

    sk=skrlst[i];
    switch (sk->stat) {
      case s_active	:
	pidlst[n]=sk->skrpid;
	n++;
	break;
      default		:
	break;
      }
    }
  if (n>0) 
    (void) rou_childkill(pidlst,n,5);
  (void) free(pidlst);
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to handle a checkrcpt		*/
/*	request, it try to find out what are the*/
/*	sender/checker related to to this	*/
/*	recipient E-mail.			*/
/*						*/
/************************************************/
static char *checkrcpt(BOXTYP *boxe,SKRTYP **skrlst,char *caller,char *rqst)

{
char *answ;
char *domain;
RCPTMSG *rm;
DOMTYP *skrs;
int phase;
int proceed;

answ=(char *)0;
domain=(char *)0;
rm=(RCPTMSG *)0;
skrs=(DOMTYP *)0;
phase=0;
proceed=true;
while (proceed==true) {
  switch (phase) {
    case  0	:		/*scanning request	*/
      if ((rm=eml_getrcptmsg(rqst))==(RCPTMSG *)0) {
	answ=strdup(eml_ascstat(BADPAR,rqst));
	proceed=false;
	}
      break;
    case  1		:	/*is there any SKR ?	*/
      domain=eml_strdomain(rm->rcpt);
      if ((skrs=eml_findskr(skrlst,domain,rm->relais))==(DOMTYP *)0){
        answ=strdup(eml_ascstat(NORELAY,rm->rcpt));
	proceed=false;
	}
      break;
    case  2		:	/*forward to main sender*/
      if ((skrs->sndr1!=(SKRTYP *)0)&&(skrs->sndr1->skrpid!=(pid_t)0)) {
	char *locmsg;
	pid_t chkr1;
	pid_t sndr2;

	chkr1=(pid_t)0;
        sndr2=(pid_t)0;
	if (skrs->chkr1!=(SKRTYP *)0) {
	  chkr1=skrs->chkr1->skrpid;
	  }
	if (skrs->sndr2!=(SKRTYP *)0)
	  sndr2=skrs->sndr2->skrpid;
        locmsg=rou_memstrcat((char *)0,"%s %s |%05d %05d",
					CKRSND,rqst,chkr1,sndr2);
        if (ipc_relaymsg(boxe,caller,skrs->sndr1->skrpid,locmsg)==0) {
          answ=strdup(eml_ascstat(ERRPROC,rqst));
	  }
	phase=999;
	} 
      break;
    case  3		:	/*sending to secondary 	*/
      if ((skrs->sndr2!=(SKRTYP *)0)&&(skrs->sndr2->skrpid!=(pid_t)0)) {
	char *locmsg;

        locmsg=rou_memstrcat((char *)0,"%s %s",CKRRLY,rqst);
        if (ipc_relaymsg(boxe,caller,skrs->sndr2->skrpid,locmsg)==0) {
          answ=strdup(eml_ascstat(ERRPROC,rqst));
	  }
	phase=999;
	} 
      break;
    case  4		:	/*errproc status	*/
      answ=strdup(eml_ascstat(NOSNDR));
      (void) alert(0,"modchkrc.c:checkrcpt, No sender defined for <%s>",
		      domain);
      break;
    default		:	/*SAFE Guard		*/
      (void) free(skrs);
      proceed=false;
      break;
    }
  phase++;
  }
return answ;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to send the SENT message to 	*/
/*	the proper checker, if there is no 	*/
/*	checker for the domain everything is	*/
/*	forgoten.				*/
/*						*/
/************************************************/
static void relaysent(BOXTYP *boxe,SKRTYP **skrlst,char *caller,char *rqst)

{
char **data;

if ((data=eml_splitmsg(rqst))!=(char **)0) {
  int i;
  SNDTYP **sndlst;

  sndlst=(SNDTYP **)0;
  for (i=0;data[i]!=(char *)0;i++) {
    char emailid[50];
    char chkname[100];
    char recipient[300];

    if (sscanf(data[i],"%95s %45s %295s",chkname,emailid,recipient)==3) {
      SNDTYP *chn;
      

      if ((chn=findsentlst(sndlst,chkname))==(SNDTYP *)0) {
	chn=(SNDTYP *)calloc(1,sizeof(SNDTYP));
	chn->chkname=strdup(chkname);
	sndlst=(SNDTYP **)rou_addveclst((void **)sndlst,(void *)chn);
        chn->lst=rou_memstrcat((char *)0,"%s %s",emailid,recipient);
	}
      else
        chn->lst=rou_memstrcat(chn->lst,"\n%s %s",emailid,recipient);
      }
    }
  if (sndlst!=(SNDTYP **)0) {
    int j;

    for (j=0;sndlst[j]!=(SNDTYP *)0;j++) {
      SKRTYP *skr;

      if ((skr=cnf_getskr(skrlst,sndlst[j]->chkname))!=(SKRTYP *)0) {
	if (skr->skrpid!=(pid_t)0) {
          char *locmsg;

          locmsg=rou_memstrcat((char *)0,"%s %s",SENT,sndlst[j]->lst);
          (void) ipc_relaymsg(boxe,caller,skr->skrpid,locmsg);
	  }
	}
      sndlst[j]->lst=rou_freestr(sndlst[j]->lst);
      sndlst[j]->chkname=rou_freestr(sndlst[j]->chkname);
      (void) free(sndlst[j]);
      }
    (void) free(sndlst);
    }
  data=rou_freestrlst(data);
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to send the UNIQID to the	*/
/*	proper checker, if there is no checker	*/
/*	for the domain a plain CMDOK is returned*/
/*	(provided data is not relevant)		*/
/*						*/
/************************************************/
static char *relayuiqid(BOXTYP *boxe,SKRTYP **skrlst,char *caller,char *rqst)

{
char *answ;
DOMTYP *skrs;
int status;
int phase;
int proceed;
char email[300];

answ=(char *)0;
skrs=(DOMTYP *)0;
status=BADPAR;
phase=0;
proceed=true;
while (proceed==true) {
  switch (phase) {
    case  0	:	/*extracting reqtest domain	*/
      if (sscanf(rqst,"%s",email)!=1) {
        (void) alert(0,"modchk.c:relayuiqid, Unable to scan <%s>",rqst);
	proceed=false;
	}
      break;
    case  1	:	/*finding checker		*/
      if ((skrs=eml_findskr(skrlst,eml_strdomain(email),false))==(DOMTYP *)0){
	status=CMDOK;	/*no checker?, it is A_OK	*/
	phase=999;
	}
      break;
    case  2	:	/*do we have an active checker	*/
      status=CMDOK;
      if ((skrs->chkr1!=(SKRTYP *)0)&&(skrs->chkr1->skrpid!=(pid_t)0)) {
        char *locmsg;

        locmsg=rou_memstrcat((char *)0,"%s %s",UNIQID,rqst);
        if (ipc_relaymsg(boxe,caller,skrs->chkr1->skrpid,locmsg)!=0) {
	  proceed=false;
	  }
	else {
	  /*we should do something if we are not able to	*/
	  /*transmit the uniqid info to checker (unlikely).	*/
	  /*Lets return CMDOK for now.				*/
	  }
	}
      break;
    default	:	/*SAFE guard			*/
      answ=strdup(eml_ascstat(status,rqst));
      proceed=false;
      break;
    }
  phase++;
  }
return answ;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to handle a checkadvice	*/
/*	purpose is to find out what are the	*/
/*	checker in charge to give avdvice and	*/
/*	dispatch subrequest. Unassigned request	*/
/*	part got a default status.		*/
/*						*/
/************************************************/
static char *checkadv(BOXTYP *boxe,SKRTYP **skrlst,char *caller,char *rqst)

{
char *answ;
DISTYP **msglst;
ADVIMSG *advimsg;

answ=(char *)0;
msglst=(DISTYP **)0;
if ((advimsg=eml_getadvimsg(rqst))!=(ADVIMSG *)0) {
  int slice;	/*number of slice for the message	*/

  slice=0;
  if (advimsg->doms!=(DOMCHK **)0) {
    int i;

    for (i=0;advimsg->doms[i]!=(DOMCHK *)0;i++) {
      DOMCHK *domloc;
      DOMTYP *skrs;

      domloc=advimsg->doms[i];
      skrs=(DOMTYP *)0;
      if ((skrs=eml_findskr(skrlst,domloc->domain,false))!=(DOMTYP *)0){
	if ((skrs->chkr1!=(SKRTYP *)0)&&(skrs->chkr1->skrpid!=(pid_t)0)) {
	  char *dmsg;

	  if ((dmsg=eml_domtomsg(domloc))!=(char *)0) {
	    DISTYP *msgck;

	    if ((msgck=getdispatch(msglst,skrs->chkr1->skrpid))==(DISTYP *)0) {
	      msgck=(DISTYP *)calloc(1,sizeof(DISTYP));
	      msgck->chkpid=skrs->chkr1->skrpid;
	      msgck->msg=strdup(advimsg->info);
	      msglst=(DISTYP **)rou_addveclst((void **)msglst,(void *)msgck);
	      slice++;
	      }
	    msgck->msg=rou_memstrcat(msgck->msg,"%s",dmsg);
	    dmsg=rou_freestr(dmsg);
	    domloc->forwarded=true;
	    }
	  (void) free(skrs);
	  }
	}
      }
    }
  /*now lets respond A_OK for none forwarded advice rqst	*/
  slice=defltadv(boxe,caller,advimsg,slice);
  if (msglst!=(DISTYP **)0) {
    int i;

    for (i=0;msglst[i]!=(DISTYP *)0;i++) {
      char *locmsg;

      locmsg=rou_memstrcat((char *)0,"%s %d/%d %s",
				     ADVICE,i+1,slice,msglst[i]->msg);
      if (ipc_relaymsg(boxe,caller,msglst[i]->chkpid,locmsg)==0) {
	/*we should do something if we are not able to transmit	*/
	/*the advice request to checker (unlikely).		*/
	(void) alert(0,"modchk.c:checkadv, unable to restransmit <%s>",locmsg);
	}
      locmsg=rou_freestr(locmsg);
      msglst[i]->msg=rou_freestr(msglst[i]->msg);
      (void) free(msglst[i]);
      }
    (void) free(msglst);
    }
  advimsg=eml_freeadvimsg(advimsg);
  }
return answ;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to wait for SMTP request and	*/
/*	forward it to the proper sender or	*/
/*	checker.				*/
/*						*/
/************************************************/
static void chkcycle(BOXTYP *boxe,AUXTYP **auxs)

{
#define	OPCY	"modchk.c:chkcycle,"

SKRTYP **skrlst;

skrlst=cnf_getskrlst(auxs);
while (quickout==false) {
  char *msgref;
  char *rqst;
  TIMESPEC timeout;

  timeout.tv_sec=1;
  timeout.tv_nsec=0;
  if (ipc_waitpost(boxe,&timeout)==false) {
    (void) checkskr(boxe,skrlst);
    continue;
    }
  if ((rqst=ipc_getpost(boxe,&msgref))!=(char *)0) {
    char *answ;
    MENUM code;

    answ=(char *)0;
    switch (code=getmsgcode(rqst)) {
      case msg_ckrcpt		:
	answ=checkrcpt(boxe,skrlst,msgref,rqst);
	break;
      case msg_advice		:
	answ=checkadv(boxe,skrlst,msgref,rqst);
	break;
      case msg_uniqid		:
	answ=relayuiqid(boxe,skrlst,msgref,rqst);
	break;
      case msg_sent		:
	(void) relaysent(boxe,skrlst,msgref,rqst);
	break;
      case msg_ckrsnd		:
      case msg_ckrrly		:
      case msg_ckrmnt		:
	(void) alert(0,"%s Unexpected message='%d'",OPCY,code);
        answ=strdup(eml_ascstat(ERRPROC));
	break;
      case msg_unknown		:
      default			:
        (void) alert(0,"%s request <%s> unknown",OPCY,rqst);
        answ=strdup(eml_ascstat(ERRPROC));
        break;
      }
    if (answ!=(char *)0) {
      (void) ipc_postreply(boxe,msgref,answ);
      msgref=rou_freestr(msgref);
      answ=rou_freestr(answ);
      }
    rqst=rou_freestr(rqst);
    }
  }
(void) dropskr(skrlst);
skrlst=cnf_freeskrlst(skrlst);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to fork a process and start	*/
/*	wait for request to be check with a	*/
/*	local/remote sendmail.			*/
/*						*/
/************************************************/
pid_t chk_manager(BOXTYP *boxe,AUXTYP **auxs)

{
pid_t child;

switch((child=rou_fork(false))) {
  case  -1      :
    (void) alert(0,"modchk.c:chk_manager, Unable to start local process "
		   "(error=<%s>)",strerror(errno));
    break;
  case   0      :
    (void) settitle("%s, Checker control",appname);
    (void) chkcycle(boxe,auxs);
    (void) exit(0);
    break;
  default       :
    break;
  }
return child;
}
