/************************************************/
/*						*/
/*      Copyright:				*/
/*		 Jean-Marc Pigeon <jmp@safe.ca>	*/
/*	Distributed under the Gnu Public	*/
/*	License, see the License file in this	*/
/*	package.				*/
/*						*/
/*	MODTRA:					*/
/*	Module level routines to handle		*/
/*	Protocol exchange with remote site.	*/
/*						*/
/************************************************/
#include	<malloc.h>
#include	<syslog.h>
#include	<string.h>
#include	<unistd.h>
#include	<errno.h>
#include	<ctype.h>
#include	<sys/file.h>
#include	"modtra.h"
#include	"lvlspm.h"
#include	"gestls.h"
#include	"gesclm.h"
#include	"gesque.h"
#include	"unieml.h"
#include	"unidns.h"
#include	"subrou.h"

/*Special journal for WEB			*/
#define	WJRL	MXJRL-1

typedef struct	{
	char *domain;	/*domain named	*/
	RCPTYP **rcpts;	/*list of rcpts	*/
	}QLSTYP;
/*

*/
/************************************************/
/*						*/
/*	Procedure to update the local .lisq file*/
/*	within the user quarantine file.	*/
/*						*/
/************************************************/
static void updlisq(char *dir,CTTYP *cta)


{
/*
LOG Format:
ID|expediteur|date|taille|sujet
*/
#define LOGFRM "%s.%02d|%s|%s|%ld|%s\n"
int taille;
char *filename;
FILE *file;

taille=strlen(dir)+150;
filename=(char *)calloc(taille,sizeof(char));
rou_mkname(filename,taille,d_spool,dir,"/",LISQ);
if ((file=fopen(filename,"a+"))==(FILE *)0)
  (void) core_dump("%s Unable open logfile <%s> (error=<%s>)",
		   "modtra.c:updlisq,",filename,strerror(errno));
(void) flock(fileno(file),LOCK_EX);
(void) fprintf(file,LOGFRM,cta->session,cta->sequence,
			   cta->evl->realfrom,
			   rou_getstrdatetime(cta->lastxchg),
			   cta->mtaille,
			   cta->evl->subject);
(void) fclose(file);	/*Dropping lock too	*/
(void) free(filename);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to return a QLSTYP pointeur	*/
/*	according domain found.			*/
/*						*/
/************************************************/
static QLSTYP *locquar(QLSTYP **qlst,char *domain)

{
QLSTYP *quar;

quar=(QLSTYP *)0;
if (qlst!=(QLSTYP **)0) {
  int i;

  for (i=0;qlst[i]!=(QLSTYP *)0;i++) {
    if (strcmp(qlst[i]->domain,domain)==0) {
      quar=qlst[i];
      break;
      }
    }
  }
return quar;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to collect all different	*/
/*	quarantined RCPT according domain name	*/
/*						*/
/************************************************/
static QLSTYP **getqualist(CTTYP *cta)

{
QLSTYP **qlst;
int i;

qlst=(QLSTYP **)0;
for (i=0;cta->rcpts[i]!=(RCPTYP *)0;i++) {
  RCPTYP *rcpt;
  QLSTYP *nqua;
  char *domain;

  rcpt=cta->rcpts[i];
  if (rcpt->status!=m_quarantine)
    continue;
  if ((domain=eml_strdomain(rcpt->rcpt))==(char *)0)
    domain=rcpt->rcpt;
  if ((nqua=locquar(qlst,domain))==(QLSTYP *)0) {
    nqua=(QLSTYP *)calloc(1,sizeof(QLSTYP));
    nqua->domain=strdup(domain);
    qlst=(QLSTYP **)rou_addveclst((void **)qlst,(void *)nqua);
    }
  nqua->rcpts=(RCPTYP **)rou_addveclst((void **)nqua->rcpts,(void *)rcpt);
  }
return qlst;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to free all memory used	*/
/*	by a QLSTYP list			*/
/*						*/
/************************************************/
static QLSTYP **freequalist(QLSTYP **qlst)

{
if (qlst!=(QLSTYP **)0) {
  int i;

  for (i=0;qlst[i]!=(QLSTYP *)0;i++) {
    qlst[i]->domain=rou_freestr(qlst[i]->domain);
    (void) free(qlst[i]->rcpts);
    }
  (void) free(qlst);
  qlst=(QLSTYP **)0;
  }
return qlst;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to check if received E-mail	*/
/*	is plagued with a virus.		*/
/*						*/
/************************************************/
static int checkvirus(CLAMTYP *clam,CTTYP *cta,char *answ,const int taille)

{
#define	VIRUS	"VIRUS=<%s>"

const char *virname;
int itsok;

itsok=true;
switch (clm_checkvirus(clam,cta->dataname,&virname)) {
  case clm_clean	:
    break;
  case clm_virus	: 
    (void) strncpy(answ,eml_ascstat(BADVIR,virname),taille);
    cta->evl->virname=strdup(virname);
    (void) journalling(false,EVJRL,VIRUS,virname);
    itsok=false;
    break;
  case clm_nocheck	:
    (void) journalling(false,EVJRL,VIRUS,"Not checked");
    break;
  }
return itsok;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to dispatch E-mail in the	*/
/*	proper quarantine area			*/
/*						*/
/************************************************/
static void sortquare(CTTYP *cta,QLSTYP **qlst)

{
#define	OPQU	"modtra.c:sortquare,"

int i;

for (i=0;qlst[i]!=(QLSTYP *)0;i++) {
  if (qlst[i]->rcpts!=(RCPTYP **)0) {
    int j;

    for (j=0;qlst[i]->rcpts[j]!=(RCPTYP *)0;j++) {
      FILE *qfile;
      char *username;
      char *ptr;
      char *qdir;
      char ext[30];
      char filename[300];

      (void) snprintf(ext,sizeof(ext),".%02d%s",cta->sequence,QUEUEEXT);
      username=strdup(qlst[i]->rcpts[j]->rcpt);
      if ((ptr=strchr(username,'@'))!=(char *)0)
        *ptr='\000';
      qdir=rou_memstrcat((char *)0,"%s%s/%s/",
				   TRIADIR,qlst[i]->domain,username);
      (void) rou_mkname(filename,sizeof(filename)-20,d_spool,qdir,"","");
       filename[strlen(filename)-1]='\000';
      (void) rou_createdirs(filename);
      (void) rou_mkname(filename,sizeof(filename),d_spool,qdir,
						  cta->session,ext);
      if ((qfile=fopen(filename,"w"))==(FILE *)0) 
        (void) core_dump("%s Unable to open <%s> (error=<%s>)",
			 OPQU,filename,strerror(errno));
      (void) fprintf(qfile,"%s %s\n",SMAILF,cta->frm->from);
      (void) fprintf(qfile,"%s %s\n",SRCPTT,qlst[i]->rcpts[j]->rcpt);
      (void) fprintf(qfile,SQUIT);
      (void) fclose(qfile);
      /*duplicating E-mail data via hard link			*/
      (void) snprintf(ext,sizeof(ext),".%02d%s",cta->sequence,EMAILEXT);
      (void) rou_mkname(filename,sizeof(filename),d_spool,qdir,
						  cta->session,ext);
      if (link(cta->dataname,filename)<0) 
        (void) core_dump("%s Unable to link E-mail data to <%s> (error=<%s>)",
			  OPQU,filename,strerror(errno));
      (void) updlisq(qdir,cta);
      qdir=rou_freestr(qdir);
      username=rou_freestr(username);
      }
    }
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to prepare the actual	E-mail	*/
/*	sending to sendmail for direct E-mail.	*/
/*						*/
/************************************************/
static void sortdirect(CTTYP *contact)

{
#define	OPDI	"modtra.c:sortdirect,"

int i;
FILE *qfile;
char ext[10];
char filename[300];

(void) snprintf(ext,sizeof(ext),".%02d%s",contact->sequence,QUEUEEXT);
(void) rou_mkname(filename,sizeof(filename),d_spool,QUEUDIR,
					    contact->session,ext);
if ((qfile=fopen(filename,"w"))==(FILE *)0)
  (void) core_dump("%s Unable to open <%s> (error=<%s>)",
		    OPDI,filename,strerror(errno));
if (contact->relayable==true)
  (void) fprintf(qfile,"%s 1\n",SRELAY);
(void) fprintf(qfile,"%s %s\n",SMAILF,contact->frm->from);
for (i=0;contact->rcpts[i]!=(RCPTYP *)0;i++) {
  switch (contact->rcpts[i]->status) {
    case m_relayed	:
    case m_direct	:
      (void) fprintf(qfile,"%s %s\n",SRCPTT,contact->rcpts[i]->rcpt);
      break;
    case m_rejected	:	/*nothing to do	*/
    case m_quarantine	:	/*nothing to do	*/
      break;
    default		:
      (void) alert(0,"%s rcpt status unkcheck '%d'",contact->rcpts[i]->status);
      break;
    }
  }
(void) fprintf(qfile,SQUIT);
(void) fclose(qfile);
/*duplicating E-mail data via hard link			*/
(void) snprintf(ext,sizeof(ext),".%02d%s",contact->sequence,EMAILEXT);
(void) rou_mkname(filename,sizeof(filename),d_spool,QUEUDIR,
						    contact->session,ext);
if (link(contact->dataname,filename)<0) 
  (void) core_dump("%s Unable to link E-mail data to <%s> (error=<%s>)",
		      OPDI,filename,strerror(errno));
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to wait for advices, as 	*/
/*	can come from differente checker, 	*/
/*	procedure wait until all advices are	*/
/*	received or a timeout is set.		*/
/*						*/
/************************************************/
static char *getadvrply(BOXTYP *boxe,char *msgref)

{
#define	OPGE	"motra.c:getadvrply,"

#define	MXWRPLY	120	/*max sec to get advice	*/
#define MXCOL	4000

char *rsp;
int maxmsg;
int gotall;
time_t start;
char collected[MXCOL];

rsp=(char *)0;
maxmsg=0;
gotall=false;
start=rou_systime();
(void) memset(collected,'\000',sizeof(collected));
while ((start+MXWRPLY)>rou_systime()) {
  char *got;
 
  if ((gotall==true)||(quickout==true))
    break;
  (void) usleep(1000000); 
  if ((got=ipc_getreply(boxe,msgref))!=(char *)0) {
    char **split;

    if ((split=eml_splitmsg(got))!=(char **)0) {
      int num;

      if ((num=rou_vecsize((void **)split))>0) {
	int status;
	int nummsg;

	num--;
	nummsg=0;
	if (sscanf(split[num],"%d %d/%d",&status,&nummsg,&maxmsg)==3) {
          int i;

	  if (maxmsg>=MXCOL)
	    core_dump("%s far too many (>%d) reply!",OPGE,MXCOL);
	  if (nummsg<1)
	    core_dump("%s error in protocol reply!",OPGE);
	  collected[nummsg-1]='A';
	  gotall=(strlen(collected)==maxmsg);
	  if (status==CMDOK) {
	    for (i=0;i<num;i++)
	      rsp=rou_memstrcat(rsp,"%s\n",split[i]);
	    }
	  }
	}
      split=rou_freestrlst(split);
      }
    got=rou_freestr(got);
    }
  }
if (rsp!=(char *)0)
  rsp=rou_memstrcat(rsp,"%d %d/%d This were my advices",
			 CMDOK,strlen(collected),maxmsg);
else {
  errno=ETIME;
  }
return rsp;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to send an advice request and	*/
/*	wait for replies.			*/
/*						*/
/************************************************/
static char *waitforadv(BOXTYP *boxe,char *msg)

{
#define	OPWA	"modtra.c:waitforadv,"
char *rsp;
char *msgref;

rsp=(char *)0;
if ((msgref=ipc_postmsg(boxe,getppid(),msg))!=(char *)0) {
  rsp=getadvrply(boxe,msgref);
  msgref=rou_freestr(msgref);
  }
else {
  (void) alert(0,"%s Unable to post message <%s> (error=<%s>)",
		  OPWA,msg,strerror(errno));
  }
if (rsp==(char *)0) {
  (void) tracing(TRALOG,0,"%s, No reply to advice request",OPWA);
  journalling(false,APJRL,"No reply to advice request to <%s>",msg);
  }
return rsp;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to adjuste eash user status	*/
/*	deciding if E-mail need to go in	*/
/*	quarantine or to be transmited direct	*/
/*	this is done by consulting remote 	*/
/*	mentor.					*/
/*						*/
/************************************************/
static void checkuserstatus(CTTYP *contact)

{
if (contact->rcpts!=(RCPTYP **)0) {
  int i;

  for (i=0;contact->rcpts[i]!=(RCPTYP *)0;i++) {
    RCPTYP *rc;

    rc=contact->rcpts[i];
    if (rc->status==m_indata) {
      rc->status=m_direct;
      rc->why=strdup("accepted"); 
      }
    }
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to do the actual E-mail	*/
/*	sorting according RCPT status.		*/
/*						*/
/************************************************/
static void sortoutemail(CTTYP *contact)

{
int i;

for (i=0;i<2;i++) {
  int count;

  count=0;
  switch(i)  {
    case 0  :	/*taking care of direct E-mail	*/
      count=eml_countadvice(contact,m_direct);
      count+=eml_countadvice(contact,m_relayed);
      if (count>0)
	(void) sortdirect(contact);
      break;
    case 1  :	/*taking care of quarantine	*/
      count=eml_countadvice(contact,m_quarantine);
      if (count>0) {
	QLSTYP **quare;

	if ((quare=getqualist(contact))!=(QLSTYP **)0) {
	  (void) sortquare(contact,quare);
	  quare=freequalist(quare);
	  }
	}
      break;
    }
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to handle RCPT checking	*/
/*	via a local (or remote) SMTP server	*/
/*						*/
/************************************************/
static int rmtchecking(TRATYP *t,char *sesid,FRMTYP *frm,char *rcpt,
		       int relayable,char **rsp)

{
#define	CKFRM	"%s %s %s %lu %s,%d %s %d"
int checkstat;
char *ptrstat;
char errstat[500];

checkstat=ERRPROC;
(void) strcpy(errstat,"");
if ((*rsp=eml_xchgmsg(t->boxe,getppid(),CKFRM,CKRCPT,frm->ipremote,
		      frm->from,frm->mtaille,
		      frm->listenip,frm->listenport,
		      rcpt,relayable))!=(char *)0) {
  int pos;
  char *ptr;

  pos=0;
  (void) que_cleanstring(*rsp);
  while ((ptr=strchr(*rsp,'\n'))!=(char *)0)
     *ptr=' ';	/*to remove any LF within the reply	*/
  (void) sscanf(*rsp,"%d%n",&checkstat,&pos);
  ptr=(*rsp)+pos;
  (void) strcpy(*rsp,ptr);
  while (**rsp==' ')
    (void) strcpy(*rsp,(*rsp)+1);
  }
ptrstat=eml_ascstat(checkstat,*rsp);
switch (checkstat) {
  case RELAYOK	:	/*RELAYOK become CMDOK		*/
  case CMDOK	:
    (void) snprintf(errstat,sizeof(errstat),"%d Address Accepted",CMDOK);
    break;
  case NOSPAMS	:
    (void) strncpy(errstat,ptrstat,sizeof(errstat));
    break;
  case BADSPF	:
  case NEWSITE	:
  case CHKUSR	:
  case ERRPROC	:
    (void) strncpy(errstat,ptrstat,sizeof(errstat));
    break;
  case REJECT	:
  default	:
    if (strlen(*rsp)>0)
      (void) snprintf(errstat,sizeof(errstat),"%d %s",checkstat,*rsp);
    else
      (void) strncpy(errstat,ptrstat,sizeof(errstat));
    break;
  }
*rsp=rou_freestr(*rsp);
*rsp=strdup(errstat);
return checkstat;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to check the advice returned	*/
/*	by the remote mentor.			*/
/*						*/
/************************************************/
static int checkadvice(CTTYP *cat,char *rsp)

{
int status;
char **data;

status=ERRPROC;
if ((data=eml_splitmsg(rsp))!=(char **)0) {
  int num;

  if ((num=rou_vecsize((void **)data))>0) {
    int max;
    int i;

    num--;
    (void) sscanf(data[num],"%d",&status);
    max=rou_vecsize((void **)cat->rcpts);
    for (i=0;i<num;i++) {
      int numrcpt;
      int locstat;
     
      (void) sscanf(data[i],"%d-%d",&locstat,&numrcpt);
      if (numrcpt<max) {
	switch (locstat) {
	  case SENADV	:
	    cat->rcpts[numrcpt]->status=m_direct;
	    break;
	  case QUAADV	:
	    cat->rcpts[numrcpt]->status=m_quarantine;
	    break;
	  case DISADV	:	/*nothing to do	*/
	    break;
	  case RLYADV	:	/*nothing to do	*/
	    break;
	  default	:
	    (void) alert(0,"modtra.c:checkadvice, Unknown status <%s>",
			    data[i]);
	    break;
	  }
	}
      }
    }
  data=rou_freestrlst(data);
  }
return status;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to log meaningful E-mail	*/
/*	header informatin to be reported in	*/
/*	local logs.				*/
/*						*/
/************************************************/
static void reporthdrs(CTTYP *cta)

{
if (cta->evl->realfrom!=(char *)0)
  (void) journalling(false,EVJRL,"E-From: %s",cta->evl->realfrom);
if (cta->evl->subject!=(char *)0)
  (void) journalling(false,EVJRL,"Subject: %s",cta->evl->subject);
if (cta->evl->msgid!=(char *)0)
  (void) journalling(false,EVJRL,"Message-Id: %s",cta->evl->msgid);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to 'force' an advice in case	*/
/*	no answer was provided by remote mentor	*/
/*						*/
/************************************************/
static void forceadvice(CTTYP *cta)

{
if (cta->rcpts!=(RCPTYP **)0) {
  int i;

  (void) journalling(false,EVJRL,"Advice localy set");
  for (i=0;cta->rcpts[i]!=(RCPTYP *)0;i++) {
    RCPTYP *rc;

    rc=cta->rcpts[i];
    switch (rc->status) {
      case m_indata	: /*Lets say Email good	*/
	rc->status=m_direct;
        break;
      case m_quarantine	:
      case m_direct	:
      case m_relayed	:
      case m_rejected	:
      case m_discarded	:
        break;
      }
    }
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to send header information	*/
/*	to remote Mentor, this will help	*/
/*	Mentor to advise clement about E-mail	*/
/*						*/
/************************************************/
static char *getadvice(TRATYP *track,CTTYP *cta)

{
#define	OPHD	"modtra.c:sendhdrs,"

char *rsp;
char *msg;
int phase;
int proceed;

rsp=(char *)0;
msg=(char *)0;
phase=0;
proceed=true;
while (proceed==true) {
  switch (phase)  {
    case  0	:	/*Message Channel		*/
      msg=rou_memstrcat(msg,"%s 1/1 ",ADVICE);
      break;
    case  1	:	/*Message Type & origine	*/
      msg=rou_memstrcat(msg,"%s %s\n",MENID,cta->session);
      msg=rou_memstrcat(msg,"%s %s\n",CLIP,cta->ipremote);
      msg=rou_memstrcat(msg,"%s %s\n",MFROM,cta->frm->from);
      break;
    case  2	:
      if (cta->evl->subject!=(char *)0)
        msg=rou_memstrcat(msg,"%s %s\n",SUBJ,cta->evl->subject);
      break;
    case  3	:
      if (cta->evl->msgid!=(char *)0)
        msg=rou_memstrcat(msg,"%s %s\n",MSGID,cta->evl->msgid);
      break;
    case  4	:
      if (cta->evl->inrplyto!=(char **)0) {
	int i;
	
	for (i=0;cta->evl->inrplyto[i]!=(char *)0;i++) 
          msg=rou_memstrcat(msg,"%s %s\n",RPLID,cta->evl->inrplyto[i]);
	}
      break;
    case  5	:
      msg=rou_memstrcat(msg,"%s %ld %ld %d %d\n",MSGINF,cta->evl->cachet,
			     cta->mtaille,cta->evl->nbrerror,cta->relayable);
      break;
    case  6	:
      if (cta->evl->realfrom!=(char *)0)
        msg=rou_memstrcat(msg,"%s %s\n",HFROM,cta->evl->realfrom);
      break;
    case  7	:
      if (cta->evl->spamlvl!=(char *)0)
        msg=rou_memstrcat(msg,"%s %s\n",SPAM,cta->evl->spamlvl);
      if (cta->evl->virname!=(char *)0) 
        msg=rou_memstrcat(msg,"%s %s\n",VIRINF,cta->evl->virname);
      break;
    case  8	:
      if (cta->evl->distlist!=(char **)0) {
	int i;
	
	for (i=0;cta->evl->distlist[i]!=(char *)0;i++)
          msg=rou_memstrcat(msg,"%s %s\n",HTO,cta->evl->distlist[i]);
	}
      break;
    case  9	:
      if (cta->rcpts!=(RCPTYP **)0) {
	int i;

	for (i=0;cta->rcpts[i]!=(RCPTYP *)0;i++) {
	  int status;
	  RCPTYP *rc;
	  char *why;

	  rc=cta->rcpts[i];
	  why="";
	  status=GIVADV;
	  switch (rc->status) {
	    case m_indata	:
	    case m_quarantine	:
	    case m_direct	:
	      break;
	    case m_relayed	:
	      status=RLYADV;
	      why=rc->why;
	      break;
	    case m_rejected	:
	      status=DISADV;
	      why=rc->why;
	      break;
	    case m_discarded	:	/*should never occure	*/
	      break;
	    }
          msg=rou_memstrcat(msg,"%s %02d <%s> %03d %s\n",
				 ADVRCPT,i,rc->rcpt,status,why);
	  }
        }
      break;
    default	:	/* SAFE guard processus done		*/
      proceed=false;
      break;
    }
  phase++;
  }
if ((rsp=waitforadv(track->boxe,msg))==(char *)0) {
  (void) alert(0,"%s Unable to get message <%s> reply , (error=<%s>)",
      		    OPHD,msg,strerror(errno));
  }
(void) free(msg);
return rsp;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to transmit en get reply	*/
/*	about a user ID. used to allow access	*/
/*	Clement WEB interface.			*/
/*						*/
/************************************************/
static int checknewid(TRATYP *t,CTTYP *c)

{
#define	OPCK	"modtra.c:checknewid,"

#define	CKIDFRM	"%s: %s"

int checkstat;
char *rsp;
int port;
char username[100];
char password[100];
char chkidsrv[100];
char errstat[400];

checkstat=ERRPROC;
rsp=(char *)0;
port=0;
(void) strcpy(username,"");
(void) strcpy(password,"");
(void) strcpy(chkidsrv,"");
if (sscanf(t->chkidsrv,"%95[^,\n\r],%d",chkidsrv,&port)!=2) {
  (void) alert(0,"%s Unable to scan chkidsrv <%s> (format incorrect)",
		  OPCK,t->chkidsrv);
  }
else {
  char *info;
  char *ptr;

  info=c->info;
  c->info=(char *)0;
  if (info!=(char *)0) {
    if ((ptr=strchr(info,':'))!=(char *)0)
      (void) strcpy(info,ptr+1);
    (void) sscanf(info,"%95s %95s",username,password);
    if ((rsp=eml_xchgmsg(t->boxe,getppid(),CKIDFRM,CHKID,info))!=(char *)0) {
      int pos;
  
      (void) que_cleanstring(rsp);
      pos=0;
      if (sscanf(rsp,"%d %n",&checkstat,&pos)==1)
        (void) strcpy(rsp,rsp+pos);
      }
    info=rou_freestr(info);
    }
  }
switch (checkstat) {
  case ERRPROC	:
    (void) snprintf(errstat,sizeof(errstat),"%s",eml_ascstat(checkstat));
    (void) alert(0,"CHECKID wrong command");
    break;
  case IDOK	:
    (void) snprintf(errstat,sizeof(errstat),"%d %s",checkstat,rsp);
    (void) journalling(true,EVJRL,"MCHKID: %s (%s)",errstat,username);
    break;
  case CMDOK	:
    (void) snprintf(errstat,sizeof(errstat),"%d %s",checkstat,rsp);
    (void) journalling(true,EVJRL,"MCHKID: %s (%s)",errstat,username);
    break;
  case BADID	:
  default	:
    (void) snprintf(errstat,sizeof(errstat),"%d %s",checkstat,rsp);
    (void) journalling(true,EVJRL,"MCHKID: %s (reject->%s, %s)",
			           errstat,username,password);
    break;
  }
rsp=rou_freestr(rsp);
(void) eml_printfcrlf(c->tls,"%s",errstat);
return ((checkstat==CMDOK)||(checkstat==IDOK));
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to handle RCPT check, first	*/
/*	check the RCPT localy and then do	*/
/*	a deep checking by asking to mentor.	*/
/*						*/
/************************************************/
static int checkrcpt(TRATYP *t,CTTYP *c)

{
#define	MXRCPT	50	/*maximum number of RCPT*/
#define	NOMX	"No MX or IP linkable to originator domain"

RCPTYP *rmt;
char *dom;
int letsroll;
long nr;
int status;
int proceed;
int phase;

rmt=(RCPTYP *)calloc(1,sizeof(RCPTYP));
rmt->status=m_indata;
rmt->rcpt=c->info;
dom=(char *)0;
c->info=(char *)0;
letsroll=true;
nr=1;
status=REJECT;
proceed=true;
phase=1;
while (proceed==true) {
  switch (phase) {
    case   0:		/*trouble, troube			*/
      proceed=false;
      break;
    case   1	: 	/*is user was already in rcpt		*/
      if (c->rcpts!=(RCPTYP **)0) {
        int i;

	for (i=0;c->rcpts[i]!=(RCPTYP *)0;i++) {
	  if (strcmp(c->rcpts[i]->rcpt,rmt->rcpt)==0) {
	    rmt->status=m_discarded;
	    rmt->smtpstatus=c->rcpts[i]->smtpstatus;
	    rmt->why=strdup(c->rcpts[i]->why);
	    break;
	    }
	  nr++;
	  }
        }
      break;
    case   2	: 	/*checking total number of RCPT		*/
      if ((c->relayable==false)&&(nr>MXRCPT)) {
	rmt->why=strdup(eml_ascstat(BADPAR,"Too many recipient specified"));
	rmt->status=m_rejected;
	}
      break;
    case   3	: 	/*no need to test if already rejected	*/
      switch (rmt->status) {
	case m_rejected		:
	case m_discarded	:
	  phase++;
	  break;
	default			:
	  break;
	}
      break;
    case   4	: {	/*checking via remote connection	*/
      char *answ;

      answ=(char *)0;
      switch ((status=rmtchecking(t,c->session,c->frm,rmt->rcpt,
				  c->authok,&answ))) {
	case CMDOK	:	/*everything OK			*/
	  break;
	case RELAYOK	:	/*remote IP is relayable	*/
	  c->relayable=true;
	  rmt->status=m_relayed;
	  status=CMDOK;		/*Lets say everything OK	*/
	  break;
	case NOMENT	:	/*mentor do care about it	*/
	  if (c->authok==true) {/*we trust remote originator	*/
	    answ=rou_freestr(answ);
	    answ=rou_memstrcat(answ,"%d Address relayable",CMDOK);
	    c->relayable=true;
	    rmt->status=m_relayed;	/*mail will be relayed	*/
	    status=CMDOK;
	    }
	  else
	    rmt->status=m_rejected;
	  break;
	case NOSPAMS	:
	  rmt->status=m_rejected;
	  break;
	case ERRPROC	:	/*we are really in deep trouble	*/
	  rmt->status=m_rejected;
	  letsroll=false;
	  quickout=true;	/*lets get out fast		*/
	  break;
	default		:	/*too bad for other status	*/
	  rmt->status=m_rejected;
	  break;
	}
      rmt->smtpstatus=status;
      rmt->why=answ;
      switch (rmt->smtpstatus) {
	case NEWSITE	:	/*no need to go further	*/
	case NOSPAMS	:	/*site is new or spammer*/
	  letsroll=false;	/*lets forget remote	*/
	  break;
	default		:	/*lets continue	session	*/
	  break;		/*nothing to do		*/
	}
      }
      break;
    case    5	:	/*checking MX for domain part 	*/
      dom=eml_strdomain(c->frm->from);
      if ((dom!=(char *)0)&&(dom!=c->frm->from)) {
	MXTYP **mxlst;
	u_int32_t **addrlist;

	mxlst=(MXTYP **)0;
	addrlist=(u_int32_t **)0;
	if ((mxlst=dns_getmx(dom))!=(MXTYP **)0) {
          mxlst=dns_cleanmx(mxlst);
	  break;	/*there is an MX		*/
	  }
        if ((addrlist=dns_getaddr(dom))!=(u_int32_t **)0) {
	  addrlist=dns_cleanaddr(addrlist);
	  break;	/*There an IP address		*/
	  }
	rmt->status=m_rejected;
	rmt->why=strdup(eml_ascstat(BADPAR,NOMX));
	letsroll=false;
        }
      break;
    default	: {	/*false safe, reporting status	*/
      int pos;
    			/*checking # already rejected	*/
      if (c->rcpts!=(RCPTYP **)0) {
	int verybad;
        int bad;
	int i;

	verybad=false;
	bad=0;	
	for (i=0;(c->rcpts[i]!=(RCPTYP *)0)&&(verybad==false);i++) {
	  switch (c->rcpts[i]->smtpstatus) {
	    case RELAYOK:	/*not to count bad	*/
	    case CMDOK	:	/*not to count bad	*/
	      break;
	    case NOSPAMS:
	    case BADSPF	:
	      if ((i==bad)&&(i>=3))
	        verybad=true;
	      break;
	    default	:
	      bad++;
	      if (bad>10)
		verybad=true;
	      break;
	    }
	  }
        if (verybad==true) {
	  rmt->why=strdup(eml_ascstat(MANYRJT));
	  letsroll=false;/*lets forget about remote	*/
	  }
        }
      pos=0;
      if (sscanf(rmt->why,"%d%n",&rmt->smtpstatus,&pos)==1) {
        (void) strcpy(rmt->why,rmt->why+pos);
	}
      (void) eml_printfcrlf(c->tls,"%d%s",rmt->smtpstatus,rmt->why);
      (void) journalling(false,EVJRL,"MRCPT: %d <%s>\n%-9s%s",
				     rmt->smtpstatus,rmt->rcpt,"",rmt->why);
      if (rmt->status==m_rejected) {
        char *ptr;

	while ((ptr=strstr(rmt->why,">, "))!=(char *)0) {
	  ptr++;	/*stripping the ATTENTION!!! part	*/
	  *ptr='\000';
          }
        (void) journalling(true,WJRL,"%s %d %s <%s> <%s> STATUS: %s",
			             "Rejected",
			             0,"Bytes",
			             c->frm->from,
			             rmt->rcpt,
			             rmt->why);
	}
      switch (rmt->status) {
	case m_discarded:
	  rmt=eml_freercpt(rmt);
	  break;
	default		:
          c->rcpts=(RCPTYP **)rou_addveclst((void **)c->rcpts,(void *)rmt);
	  break;
	}
      if (letsroll==false)
        (void) journalling(false,EVJRL,"STOP: disconnecting session");
      proceed=false;
      }
      break;
    }
  phase++;
  }
return letsroll;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to write E-mail report within	*/
/*	WEB area, such it can be search and 	*/
/*	displayed Cvia the customer browser.	*/
/*						*/
/************************************************/
static void webreport(CTTYP *cta)

{
#define	OPWE	"modtra.c:webreport"

if (cta->rcpts!=(RCPTYP **)0) {
  char *subject;
  char *unit;
  char *mailfrom;
  u_long ltaille;
  int i;
  
  ltaille=cta->mtaille;
  unit="Bts";
  if (ltaille>1024) {
    unit="KBts";
    ltaille/=1024;
    }
  if (ltaille>1024) {
    unit="MBts";
    ltaille/=1024;
    }
  ltaille++;
  mailfrom=cta->evl->realfrom;
  if (mailfrom==(char *)0)
     mailfrom=cta->frm->from;
  if (cta->evl->subject!=(char *)0) 
    subject=strdup(cta->evl->subject);
  else
    subject=strdup("");
  for (i=0;cta->rcpts[i]!=(RCPTYP *)0;i++) {
    RCPTYP *rc;
    char *code;

    rc=cta->rcpts[i];
    code="ERRPROC";
    switch (rc->status) {
      case m_direct	:
        code="Direct";
        break;
      case m_relayed	:
        code="Relayed";
        break;
      case m_rejected	:
        code="Rejected";
        break;
      case m_quarantine	:
        code="Quarantine";
        break;
      default		:
        core_dump("%s Unknown status '%d' for user <%s>",
		   OPWE,rc->status,rc->rcpt);
        break;
      }
    (void) journalling(true,WJRL,"%s %d %s <%s> <%s> %s",
			          code,
			          ltaille,unit,
			          mailfrom,
			          rc->rcpt,
			          subject);
    }
  subject=rou_freestr(subject);
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to write E-mail report within	*/
/*	E-mail journal and transmit the info	*/
/*	to remote checker.			*/
/*						*/
/************************************************/
static void logreport(CTTYP *cta)

{
#define	OPSE	"modtra.c:logreport"
#define	RCFM	"RCPT: %10s <%s>"

if (cta->rcpts!=(RCPTYP **)0) {
  int i;

  for (i=0;cta->rcpts[i]!=(RCPTYP *)0;i++) {
    RCPTYP *rc;
    char *code;

    rc=cta->rcpts[i];
    code="ERRPROC";
    switch (rc->status) {
      case m_direct	:
        code="Direct";
        break;
      case m_relayed	:
        code="Relayed";
        break;
      case m_rejected	:
        code="Rejected";
        break;
      case m_quarantine	:
        code="Quarantine";
        break;
      default		:
        core_dump("%s Unknown status '%d' for user <%s>",
		   OPSE,rc->status,rc->rcpt);
        break;
      }
    (void) journalling(false,EVJRL,RCFM,code,rc->rcpt);
    }
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to dispatch data according	*/
/*	information collected via Mentor.	*/
/*						*/
/************************************************/
static void dispatchdata(TRATYP *track,CTTYP *cta)

{
#define GOFRM	"%d %s-%02d Message accepted for delivery"

int phase;
int proceed;
char answer[300];

phase=1;
proceed=true;
(void) snprintf(answer,sizeof(answer),GOFRM,CMDOK,cta->session,cta->sequence);
while (proceed==true) {
  switch (phase) {
    case   0	:	/*we have a big trouble			*/
      if (cta->rcpts!=(RCPTYP **)0) {
	int i;

	for (i=0;cta->rcpts[i]!=(RCPTYP *)0;i++) {
	  RCPTYP *rcp;

	  rcp=cta->rcpts[i];
	  if (rcp->status==m_indata) {
	    rcp->status=m_rejected;
	    rcp->smtpstatus=MALABRT;
	    rcp->why=strdup(cta->info);
	    }
	  }
	}
      phase=999;
      break;
    case   1	:	/*extracting the header			*/
      if ((cta->evl=hdr_parseheader(cta->headers))==(EVLTYP *)0) {
	cta->info=rou_freestr(cta->info);
	cta->info=strdup("Incorrect E-mail Header");
	(void) snprintf(answer,sizeof(answer),eml_ascstat(MALABRT,cta->info));
	phase=-1;	/*big trouble				*/
	}
      else 
	(void) reporthdrs(cta);
      break;
    case   2	:	/*checking if E-mail is a virus		*/
      if (checkvirus(track->clam,cta,answer,sizeof(answer))==false) {
	if (cta->rcpts!=(RCPTYP **)0) {
	  int i;

	  for (i=0;cta->rcpts[i]!=(RCPTYP *)0;i++) {
	    switch (cta->rcpts[i]->status) {
	      case m_indata	:
	      case m_relayed	:
	        cta->rcpts[i]->status=m_rejected;
		break;
	      default		:
		break;
              }
	    } 
	  }
        }
      break;
    case   3	: 	/*checking spam level 			*/
      cta->evl->spamlvl=spm_checklevel(cta->dataname);
      if (cta->evl->spamlvl!=(char *)0)
        (void) journalling(false,EVJRL,"Spam-lvl: %s",cta->evl->spamlvl);
      break;
    case   4	: 	/*Displaying the MSG Info 		*/
      (void) journalling(false,EVJRL,"%s: size=%ld,n_error=%d",
			       MSGINF,cta->mtaille,cta->evl->nbrerror);
      break;
    case   5	: 	/*checking bulk level 			*/
      break;
    case   6	: {	/*Preparing and sending EVL report	*/
      char *rsp;

      if ((rsp=getadvice(track,cta))!=(char *)0) {
        int restat;

        if ((restat=checkadvice(cta,rsp))!=CMDOK) {
	  cta->info=rou_freestr(cta->info);
	  cta->info=strdup(rsp);
	  (void) snprintf(answer,sizeof(answer),"%d %s",restat,rsp);
	  phase=-1;	/*trouble trouble			*/
	  }
        rsp=rou_freestr(rsp);
	}
      else
	(void) forceadvice(cta);
      }
      break;
    case   7	:	/*final touch to user status		*/
      (void) checkuserstatus(cta);
      break;
    case   8	:	/*making actual E-mail dispatching	*/
      (void) sortoutemail(cta);
      break;
    default	:	/*lets report finding			*/
      (void) logreport(cta);
      if (cta->evl!=(EVLTYP *)0)
        (void) webreport(cta);
      (void) eml_printfcrlf(cta->tls,answer);
      proceed=false;
      break;
    }
  phase++;
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to handle top level SMTP	*/
/*	protocol with a remote client.		*/
/*						*/
/************************************************/
static void gosmtp(TRATYP *track,TLSFILE *tls,char *ipnum,int port)

{
CTTYP *contact;
int proceed;

contact=eml_gotcontact(tls);
(void) eml_setsession(tls,contact);
contact->listenip=strdup(ipnum);
contact->listenport=port;
contact->attime=strdup(rou_getstrtime(rou_systime()));
*(strrchr(contact->attime,':'))='\000';	/*removing seconds display	*/
track->clam=clm_refreshtree(track->clam);
contact->clamvers=strdup(clm_getversdate(track->clam));
(void) setjournal(WJRL,d_web,appname);
(void) eml_settitle(contact);
(void) eml_printfcrlf(tls,"%d %s ESMTP %s-%s; %s",
                          SIGNON,contact->locname,appname,
		          rou_getvers(),rou_ascsystime(rou_systime()));
proceed=true;
while ((contact=eml_domails(contact))!=(CTTYP *)0) {
  switch (contact->code) {
    case s_rcpt		:
      proceed=checkrcpt(track,contact);
      break;
    case s_auth	:
      if ((proceed=checknewid(track,contact))==true)
	contact->authok=true;
      break;
    case s_chkid	:
      proceed=checknewid(track,contact);
      break;
    case s_data		:
      (void) dispatchdata(track,contact);
      (void) eml_resetcontact(contact);
      break;
    case s_reset	:
      (void) eml_resetcontact(contact);
      (void) eml_printfcrlf(contact->tls,"%d Reset state",CMDOK);
      break;
    case s_quit		:
      (void) eml_printfcrlf(contact->tls,eml_ascstat(QUITOK,contact->locname));
			/*no BREAK		*/
    case s_timeout	:
      proceed=false;
      break;
    case s_unknown	:
    default		:
      (void) alert(0,"modtra.c:gosmtp, code '%d' unexpected (bug?)",
		      contact->code);
      break;
    }
  if (proceed==false)
    break;
  }
contact=eml_itsover(contact);
(void) ipc_purgemsgs(track->boxe,getpid());
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to wait for a connection and	*/
/*	dispatch contact according channel 	*/
/*	protocol.				*/
/*						*/
/************************************************/
int tra_tracking(TRATYP *track,LSTTYP *channel)

{
#define	OPTR	"modtra.c:tra_tracking,"

int count;
TLSFILE *tls;

count=0;
if ((tls=tls_accept(channel))!=(TLSFILE *)0) {
  switch (channel->proto) {
    case p_msmtp	:
      (void) gosmtp(track,tls,channel->ipnum,channel->port);
      count++;
      break;
    case p_undef	:
      (void) alert(0,"%s undefined protocol",OPTR);
      break;
    }
  (void) tls_close(tls);
  }
return count;
}
