/************************************************/
/*						*/
/*      Copyright:				*/
/*		 Jean-Marc Pigeon <jmp@safe.ca>	*/
/*	Distributed under the Gnu Public	*/
/*	License, see the License file in this	*/
/*	package.				*/
/*						*/
/*	MODTRP:					*/
/*	Module level routines to handle		*/
/*	local respond to Remote Mentor regarding*/
/*	Local user (in case there is a 		*/
/*	'keeplocal' definition within  the file	*/
/*	clement.conf.				*/
/*						*/
/************************************************/
#include	<unistd.h>
#include	<stdlib.h>
#include	<string.h>
#include	<syslog.h>
#include	<errno.h>
#include	"subrou.h"
#include	"unidev.h"
#include	"gesque.h"
#include	"gescnf.h"
#include	"lvleml.h"
#include	"modtrp.h"

typedef	enum	{	/*SMTP vocabulary code	*/
	t_ehlo,		/*code extension	*/
	t_id,		/*sending the ID	*/
	t_resend,	/*send Quarantined Email*/
	t_remove,	/*delete   ''      Email*/
	t_addalias,	/*adding an aliase	*/
	t_remalias,	/*removing an aliases	*/
	t_addusr,	/*adding a user		*/
	t_remusr,	/*removing an user	*/
	t_deldom,	/*deleting a domain	*/
	t_tls,		/*Transport Socket Layer*/
	t_reset,	/*reset current stat	*/ 
	t_noop,		/*operation NOOP	*/
	t_quit,		/*abort connection	*/
	t_unknown	/*unknown code		*/
	}CODTYP;

static VOCTYP trvoca[]={
		{t_ehlo,"EHLO",(const void *)0},
		{t_tls,"STARTTLS",(const void *)0},
		{t_id,"CKID",(const void *)0},
		{t_resend,"RESEND",(const void *)0},
		{t_remove,"REMOVE",(const void *)0},
		{t_addalias,"ADDALIAS",(const void *)0},
		{t_remalias,"REMALIAS",(const void *)0},
		{t_addusr,ADDUSR,(const void *)0},
		{t_remusr,REMUSR,(const void *)0},
		{t_deldom,DELDOM,(const void *)0},
		{t_reset,"RSET",(const void *)0},
		{t_noop,"NOOP",(const void *)0},
		{t_quit,"QUIT",(const void *)0},
		{t_unknown,(char *)0,(const void *)0}
		};
/*

*/
/*

*/
/************************************************/
/*						*/
/*	Procedure to handle Remote Mentor	*/
/*	Request to know if an user is existing.	*/
/*						*/
/************************************************/
static void gotrequest(TLSFILE *tls,char *ipnum,int port)

{
#define	MXTIME	120	/*120 second time out	*/
#define MXLINE	300	/*line max size		*/

CTTYP *cta;
int ckid;
CNFTYP *cnf;
int proceed;

cta=eml_gotcontact(tls);
ckid=false;
if ((cnf=cnf_readconfig())==(CNFTYP *)0) {
  (void) alert(0,"modtrp.c:gotrequest, Unable to load configuration!");
  return;
  }
(void) eml_setsession(tls,cta);
cta->listenip=strdup(ipnum);
cta->listenport=port;
cta->attime=strdup(rou_getstrtime(rou_systime()));
*(strrchr(cta->attime,':'))='\000';	/*removing seconds display	*/
(void) journalling(true,APJRL,"Transponder got contact from <%s/[%s]>",
			       cta->peername,cta->ipremote);
(void) eml_settitle(cta);
(void) eml_printfcrlf(tls,"%d %s TRSPD %s-%s; %s",
                          SIGNON,cta->locname,appname,
		          rou_getvers(),rou_ascsystime(rou_systime()));
proceed=true;
while (proceed==true) {
  VOCTYP *voc;
  int efface;
  MNGTYP code;
  char buffer[MXLINE];

  efface=true; 
  code=m_add;
  if (tls_waitforaction(MXTIME,buffer,sizeof(buffer),cta->tls)==(char *)0) {
    cta->code=s_timeout;
    break;
    }
  (void) que_cleanstring(buffer);
  cta->code=s_timeout;
  voc=rou_getvoca(trvoca,buffer);
  if ((voc==(VOCTYP *)0)||(voc->code==t_unknown)) {
    (void) eml_printfcrlf(cta->tls,eml_ascstat(SYNERR,buffer));
    break;
    }
  switch (voc->code) {
    case t_noop		:
      (void) eml_printfcrlf(cta->tls,"%d OK",CMDOK);
      break;
    case t_reset	:
      (void) eml_printfcrlf(cta->tls,"%d Reset state",CMDOK);
      (void) eml_resetcontact(cta);
      ckid=false;
      break;
    case t_id		:
      if (strcmp(cnf->dfltcode,buffer)==0)
	ckid=true;
      break;
    case t_resend	:
      efface=false;	/*NO BREAK	*/
    case t_remove	:
      if (ckid==true) {
        char email[MXLINE];
        char emid[MXLINE];

        if (sscanf(buffer,"%s %s",email,emid)==2)  
          (void) que_mngquarantine(email,emid,efface);
        (void) eml_printfcrlf(cta->tls,"%d",CMDOK);
        }
      else
        (void) eml_printfcrlf(cta->tls,"%d",BADID);
      break;
    case t_remalias	:
      code=m_rem;	/*NO BREAK	*/
    case t_addalias	:
      if (ckid==true) {
	int status;
	char domain[MXLINE];
	char name[MXLINE];
	char aliases[MXLINE];

	status=CMDABRT;
        if (sscanf(buffer,"%s %s %s",domain,name,aliases)==3) {
	  status=CMDOK;
          (void) que_mngaliases(code,domain,name,aliases);
	  }
        (void) eml_printfcrlf(cta->tls,"%s",eml_ascstat(status,buffer));
        }
      else
        (void) eml_printfcrlf(cta->tls,"%d",BADID);
      break;
    case t_deldom	:
      if (ckid==true) {
	(void) que_deldom(buffer);
        (void) eml_printfcrlf(cta->tls,"%d",CMDOK);
	}
      else
        (void) eml_printfcrlf(cta->tls,"%d",BADID);
      break;
    case t_remusr	:
      code=m_rem;	/*NO BREAK	*/
    case t_addusr	:
      if (ckid==true) {
	int status;
	PASTYP *pi;

	status=BADPAR;
	if ((pi=que_getusrinfo(buffer))!=(PASTYP *)0) {
          (void) que_mngemails(code,pi);
	  status=CMDOK;
          }
        (void) eml_printfcrlf(cta->tls,"%s",eml_ascstat(status,buffer));
        }
      else
        (void) eml_printfcrlf(cta->tls,"%d",BADID);
      break;
    case t_tls		:
      (void) eml_printfcrlf(cta->tls,"%d Ready to start TLS",SIGNON);
      (void) tls_gotls(cta->tls,true);
      break;
    case t_ehlo		:
      if (eml_checkhelo(cta,buffer,'-')==true) {
        (void) eml_printfcrlf(cta->tls,"%d STARTTLS",CMDOK);
	}
      break;
    case t_quit		:
      (void) eml_printfcrlf(cta->tls,eml_ascstat(QUITOK,cta->locname));
      proceed=false;
      break;
    }
  }
(void) journalling(true,APJRL,"Transponder contact lost with  <%s/[%s]>",
			       cta->peername,cta->ipremote);
cta=eml_itsover(cta);
cnf=cnf_freeconfig(cnf);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to detect remote Mentor	*/
/*	connection.				*/
/*						*/
/************************************************/
static void transponder(LSTTYP *channel)

{
#define	OPTA	"modtrp.c:transponder,"

TLSFILE *tls;

if ((tls=tls_accept(channel))!=(TLSFILE *)0) {
  switch (channel->proto) {
    case p_msmtp	:
      (void) gotrequest(tls,channel->ipnum,channel->port);
      break;
    case p_undef	:
    default		:
      (void) alert(0,"%s undefined protocol",OPTA);
      break;
    }
  (void) tls_close(tls);
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to start an SMTP connection	*/
/*	listening connection for remote Mentor	*/
/*	to check local user existance,		*/
/*						*/
/************************************************/
static void trpcycle(char *interface)

#define	OPTR	"modtrp.c:trpcycle,"

{
LSTTYP **look;

look=(LSTTYP **)0;
if ((look=dev_getlst(look,p_msmtp,interface))!=(LSTTYP **)0) {
  (void) journalling(true,APJRL,"Local transponder <%s> ready",interface);
  while (quickout==false) {
    (void) dev_mullisten(look);
    (void) transponder(look[0]);
    }
  (void) dev_mulclose(look);
  }
(void) sleep(1);	/*to avoid avalanche	*/
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to fork a process and start	*/
/*	wait for request to be check with a	*/
/*	local/remote sendmail.			*/
/*						*/
/************************************************/
pid_t trp_manager(char *interface)

{
pid_t child;

switch((child=rou_fork(false))) {
  case  -1      :
    (void) alert(0,"modtrp.c:trp_manager, Unable to start process "
		   "(error=<%s>)",strerror(errno));
    break;
  case   0      :
    (void) settitle("%s, Transponder on <%s>",appname,interface);
    (void) trpcycle(interface);
    (void) exit(0);
    break;
  default       :
    break;
  }
return child;
}
